/*
qpick.c  Version 0.2.0  Little Lotto Quick Pick Drawings
Copyright (C) 2002-2005  dondalah@ripco.com (Dondalah)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License as
published by the Free Software Foundation; either version 2 of
the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to:

	Free Software Foundation, Inc.
	59 Temple Place - Suite 330
	Boston, MA  02111-1307, USA.
*/

/*
Subroutine call:
    	int select;
    	double odds;
    	double getlot(int select);
    	odds = getlot(select);
*/

/* If the chi-square test fails, repeat your test. */
/* The chi-square usually passes in the next test. */
/* If the chi-square fails consistently, then the */
/* program has to be reviewed for correctness. */

/* The higher the number of iterations, the more */
/* likely the chi-square test will pass.  The chi-square */
/* test needs to have its categories filled with at */
/* least a 10 count in order to pass consistently. */
/* With 100000 iterations, the chi-square test passes */
/* consistently. */

/* See the documentation in lotprob.c regarding the */
/* calls to getlot.c. */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "rnd.h"

#define THIRTYNINE 39
#define FIVE 5

void putstx(pgm)
char *pgm;
   {
   fprintf(stderr,"Usage: %s iterations\n", pgm);
   fprintf(stderr,"Where iterations is 1 - 999999999\n");
   fprintf(stderr,"Example: %s 100000\n", pgm);
   exit(1);
   } /* putstx */

int main(argc,argv)
int argc;
char **argv;
   {
   int i,max39,indx;
   unsigned char *p,*q,*r;
   unsigned char *gameptr;
   unsigned char *trialptr;
   unsigned char thirtynine[64];
   unsigned char winners[64];
   unsigned char game[16];
   unsigned char trial[16];
   int tmp;
   int totmtch,won;
   int totplay;
   int maxplay;
   double prob,dblplay;
   double winning,totprize,netearn;
   double winpool;
   double *prizeptr;
   double prize[16];
   double odds,sum,totbingo,chisq;
   double totprob;
   double expected;
   double totcost;
   double getlot(int select);
   double *bingoptr;
   double bingo[16];
   if (argc != 2) putstx(*argv);
   maxplay = atoi(*(argv+1));
   if (maxplay < 1 || maxplay > 999999999)
      {
      fprintf(stderr,"Invalid iterations\n");
      putstx(*argv);
      }
   sd = (unsigned char *) rndinit();
   dblplay = (double) maxplay;
   totcost = dblplay * 1.0;
   winpool = totcost * 0.5;
   prizeptr = (double *) prize;
   *prizeptr++ = 100000.00;
   *prizeptr++ = 100.00;
   *prizeptr++ = 10.00;
   *prizeptr   = 1.00;
   p = (unsigned char *) thirtynine;
   q = (unsigned char *) thirtynine + THIRTYNINE;
   i = 0;
   while (p < q)
      *p++ = (unsigned char) ++i;
   *p = (unsigned char) '\0';
   p = (unsigned char *) winners;
   q = (unsigned char *) winners + THIRTYNINE;
   while (p < q)
      *p++ = (unsigned char) '\0';
   p = (unsigned char *) thirtynine;
   q = (unsigned char *) thirtynine + THIRTYNINE - 1;
   r = (unsigned char *) winners;
   gameptr = (unsigned char *) game;
   i = 0;
   max39 = THIRTYNINE;
   while (i < FIVE)
      {
      indx = rndnum(max39,sd);
      tmp = *(p+indx);
      *(r+tmp)   = (unsigned char) 255;
      *gameptr++ = (unsigned char) tmp;
      *(p+indx) = *q;
      *q-- = '\0';
      max39--;
      i++;
      } /* pick 5 balls without replacement */
   printf("Little Lotto:  ");
   gameptr = (unsigned char *) game;
   i = FIVE;
   while (i--)
      {
      printf("%2d ", (int) *gameptr++);
      } /* print 5 winning balls */
   putchar('\n');
   putchar('\n');
   printf("Match      Wins          Prize    "
      "   Expected           Odds\n");
   bingoptr = (double *) bingo;
   i = 4;
   while (i--) *bingoptr++ = 0.0;
   bingoptr = (double *) bingo;
   totplay = maxplay;
   while (totplay--)
      {
      p = (unsigned char *) thirtynine;
      q = (unsigned char *) thirtynine + THIRTYNINE;
      i = 0;
      while (p < q)
         *p++ = (unsigned char) ++i;
      *p = (unsigned char) '\0';
      totmtch = 0;
      p = (unsigned char *) thirtynine;
      q = (unsigned char *) thirtynine + THIRTYNINE - 1;
      r = (unsigned char *) winners;
      trialptr = (unsigned char *) trial;
      max39 = THIRTYNINE;
      i = FIVE;
      while (i--)
         {
         indx = rndnum(max39,sd);
         tmp = (int) *(p+indx);
         *trialptr++ = (unsigned char) tmp;
         if (*(r+tmp) == 255) totmtch++;
         *(p+indx) = *q;
         *q-- = '\0';
         max39--;
         } /* pick 5 balls without replacement */
      if (totmtch == FIVE)
         won = 0;
      else if (totmtch == 4)
         won = 1;
      else if (totmtch == 3)
         won = 2;
      else if (totmtch == 2)
         won = 3;
      else won = 4;
      *(bingoptr+won) += 1.0;
      } /* tst loop maxplay times */

   bingoptr = (double *) bingo;
   prizeptr = (double *) prize;
   totprize = 0.0;
   /* Grand Prize */
   if (*bingoptr > 0.0)
      {
      winning = *prizeptr * *bingoptr;
      totprize += winning;
      } /* if won */
   else winning = 0.0;
   prob = getlot(5);
   expected = dblplay / prob;
   printf("  5   %9.0f  %13.2f  %13.5f   %12.1f\n",
      *bingoptr, winning, expected, prob);
   bingoptr++;
   prizeptr++;

   /* Second Prize */
   if (*bingoptr > 0.0)
      {
      winning = *prizeptr * *bingoptr;
      totprize += winning;
      } /* if won */
   else winning = 0.0;
   prob = getlot(4);
   expected = dblplay / prob;
   printf("  4   %9.0f  %13.2f  %13.2f   %12.1f\n",
      *bingoptr, winning, expected, prob);
   bingoptr++;
   prizeptr++;

   /* Third Prize */
   if (*bingoptr > 0.0)
      {
      winning = *prizeptr * *bingoptr;
      totprize += winning;
      } /* if won */
   else winning = 0.0;
   prob = getlot(3);
   expected = dblplay / prob;
   printf("  3   %9.0f  %13.2f  %13.2f   %12.1f\n",
      *bingoptr, winning, expected, prob);
   bingoptr++;
   prizeptr++;

   /* Fourth Prize */
   if (*bingoptr > 0.0)
      {
      winning = *prizeptr * *bingoptr;
      totprize += winning;
      } /* if won */
   else winning = 0.0;
   prob = getlot(2);
   expected = dblplay / prob;
   printf("  2   %9.0f  %13.2f  %13.2f   %12.1f\n",
      *bingoptr, winning, expected, prob);
   bingoptr++;
   prizeptr++;

   bingoptr = (double *) bingo;
   sum = totbingo = totprob = 0.0;
   odds = getlot(5);
   totprob += (1.0 / odds);
   sum += ((*bingoptr) * (*bingoptr) * odds);
   totbingo += *bingoptr;
   bingoptr++;

   odds = getlot(4);
   totprob += (1.0 / odds);
   sum += ((*bingoptr) * (*bingoptr) * odds);
   totbingo += *bingoptr;
   bingoptr++;

   odds = getlot(3);
   totprob += (1.0 / odds);
   sum += ((*bingoptr) * (*bingoptr) * odds);
   totbingo += *bingoptr;
   bingoptr++;

   odds = getlot(2);
   totprob += (1.0 / odds);
   sum += ((*bingoptr) * (*bingoptr) * odds);
   totbingo += *bingoptr;

   sum += ((dblplay - totbingo)
      * (dblplay - totbingo)
      / (1.0 - totprob));

   expected = dblplay / (1.0 / (1.0 - totprob));
   printf("  0   %9.0f  %13.2f  %13.2f   %21.10f\n",
      (dblplay - totbingo),
      0.0,
      expected,
      (1.0 / (1.0 - totprob)));
   printf("        -------      ---------\n");
   printf("Total %9.0f  %13.2f\n\n",
      dblplay, totprize);
   printf("Total Cost       %13.2f\n\n",
      totcost);
   netearn = totprize - totcost;
   printf("Net Income       %13.2f\n\n",
      netearn);
   chisq = (sum / dblplay) - dblplay;
   printf("Chi-square %f\n", chisq);
   printf("Range at 95 percent "
      "is 0.484419 to 11.1433\n");
   free(sd);
   return(0);
   } /* main */
